//=============================================================================
// TCOUtils.
// Description: --
// 
// Date: 2016-2021
// ©copyright Francesco Biscazzo. All rights reserved.
//=============================================================================
class TCOUtils extends Info;

static function String getPackageName(coerce String pkgClassName) {
	return left(pkgClassName, inStr(pkgClassName, "."));
}

static function String getClassName(coerce String pkgClassName) {
	return right(pkgClassName, len(pkgClassName) - (inStr(pkgClassName, ".") + 1));
}

/*
 * Return:
 * -1 - if A is not relationed to B.
 * 0 - if A is the same class of B.
 * 1 - if A is child of B;
 * 2 - if A is parent of B;
 */
static function int getClassesRelation(class A, class B, optional bool bNoSubClass) {
	if (A == B)
		return 0;
	
	if (!bNoSubClass) {
		if (classIsChildOf(A, B))
			return 1;
		else if (classIsChildOf(B, A))
			return 2;
	}
	
	return -1;
}

/*
 * Return the occurrences of substrings separated by @div within @str.
 * NOTE: Escape characters are not implemented.
 *
 * @params = the string containing all the parameters.
 * @bAtLeastOne = if true and @params is empty, 1 will be returned, otherwise 0 will be returned.
 * @div = the string that separates parameters.
 */
static function int getParametersCount(String params, optional bool bAtLeastOne, optional String div) {
	local int divCount, divIdx;
   
   if (div == "")
		div = " ";
	
	if (bAtLeastOne || (!bAtLeastOne && (params != ""))) {
		do {
			divIdx = inStr(params, div);
			if (divIdx != -1) {
				divCount++;
				params = right(params, len(params) - (divIdx + 1));
			}
		} until(divIdx == -1);
		// There must be 2 parameters for each divider.
		divCount++;
	}
	
	return divCount;
}

/*
 * Get the @params substring that starts from the @paramNum occurrence of @div or at the begin of the string and ends at the @paramNum+1 occurrence of @div or at the end of the string.
 * NOTE: Escape characters are not implemented.
 *
 * @params = the string containing all the parameters.
 * @paramNum = the parameter number.
 * @div = the string that separates parameters.
 */
static function String findParameter(String params, int paramNum, optional String div) {
	local String param;
	local int i;
	local int divIdx;
	
	if (div == "")
		div = " ";
		
	if (paramNum >= 0) {
		param = params;
		for (i = 0; i <= paramNum; i++) {
			divIdx = inStr(param, div);
			
			if (i != paramNum) {
				if (divIdx == -1)
					// Not enough parameters, return an empty string.
					divIdx = len(param);
				// Get the substring that starts from the found div.
				param = right(param, len(param) - (divIdx + 1));
			} else {
				if (divIdx != -1)
					// There are additional parameters after this one, get rid of them.
					param = left(param, divIdx);
			}
		}
	}
	
	return param;
}

static function int parseLogic(coerce String logic, String falseKeywords, String trueKeywords, String toggleKeywords, optional byte keywordsSeparator, optional bool currentValue, optional bool bCaseSensitive) {
	local String sep;
	
	if (keywordsSeparator == 0)
		keywordsSeparator = asc(",");
	sep = chr(keywordsSeparator);
	falseKeywords = sep$falseKeywords$sep;
	trueKeywords = sep$trueKeywords$sep;
	toggleKeywords = sep$toggleKeywords$sep;
	logic = sep$logic$sep;
	if (!bCaseSensitive) {
		falseKeywords = caps(falseKeywords);
		trueKeywords = caps(trueKeywords);
		toggleKeywords = caps(toggleKeywords);
		logic = caps(logic);
	}
	
	if (inStr(falseKeywords, logic) != -1)
		return 0;
	else if (inStr(trueKeywords, logic) != -1)
		return 1;
	else if (inStr(toggleKeywords, logic) != -1)
		return int(!currentValue);
	
	return -1;
}

static function bool isServerPackage(Actor cmdExecutor, String pkgName) {
	local String serverPackagesStr;

	serverPackagesStr = cmdExecutor.consoleCommand("get ini:Engine.Engine.GameEngine ServerPackages");
	return inStr(serverPackagesStr, "\""$pkgName$"\"") != -1;
}

static function logStringList(String list[255], optional int countLimit, optional bool bAllowZeroCountLimit, optional String listName, optional Pawn receiver) {
	local int i;

	if (!bAllowZeroCountLimit && (countLimit == 0))
		countLimit = arrayCount(list);

	for (i = 0; i < countLimit; i++)
		if (list[i] != "") {
			if (listName == "")
				checkLog(list[i], receiver);
			else
				checkLog(listName$"["$i$"]="$list[i], receiver);
		} else {
			break;
		}
}

static function bool isClassInList(class actorClass, class list[255], optional int countLimit, optional bool bAllowZeroCountLimit) {
	local bool bResult;
	local int i;
	
	if (!bAllowZeroCountLimit && (countLimit == 0))
		countLimit = arrayCount(list);
	
	for (i = 0; i < countLimit; i++) {
		bResult = class'TCOUtils'.static.getClassesRelation(actorClass, list[i]) != -1;
		if (bResult)
			break;
	}
		
	return bResult;
}

static function bool isStringInList(coerce String str, String list[255], optional int countLimit, optional bool bAllowZeroCountLimit, optional bool bCaseSensitive) {
	local bool bResult;
	local int i;
	
	if (!bAllowZeroCountLimit && (countLimit == 0))
		countLimit = arrayCount(list);
	
	for (i = 0; i < countLimit; i++) {
		if (!bCaseSensitive)
			bResult = list[i] ~= str;
		else
			bResult = list[i] == str;
		if (bResult)
			break;
	}
		
	return bResult;
}

static function bool isNameInList(name n, name list[255], optional int countLimit, optional bool bAllowZeroCountLimit) {
	local bool bResult;
	local int i;
	
	if (!bAllowZeroCountLimit && (countLimit == 0))
		countLimit = arrayCount(list);
	
	for (i = 0; i < countLimit; i++) {
		bResult = list[i] == n;
		if (bResult)
			break;
	}
		
	return bResult;
}

// Outs the camera location based on the center of the player.
static function calcPlayerViewFromCenter(PlayerPawn pp, out actor viewActor, out vector cameraLocation, out rotator cameraRotation) {
	pp.playerCalcView(viewActor, cameraLocation, cameraRotation);
	// Revert some extra calculations added by PlayerPawn.playerCalcView().
	if (pp.viewTarget != None) {
		if (pp.viewTarget.isA('Pawn'))
			if (!pp.bBehindView)
				cameraLocation.z -= Pawn(pp.viewTarget).eyeHeight;
	} else {
		if (!pp.bBehindView) {
			cameraLocation.z -= pp.eyeHeight;
			cameraLocation -= pp.walkBob;
		}
	}
}

static function String netGetPropertyText(Object obj, String propName) {
	local String value;
	local name oldRole;
	
	if (obj.isA('Actor')) {
		oldRole = getEnum(Enum'ENetRole', Actor(obj).role);
		Actor(obj).role = Actor(obj).ENetRole.ROLE_Authority;
	}
	value = obj.getPropertyText(propName);
	if (obj.isA('Actor'))
		obj.setPropertyText("role", String(oldRole));
	
	return value;
}

static function netSetPropertyText(Object obj, String propName, coerce String propValue) {
	local name oldRole;
	
	if (obj.isA('Actor')) {
		oldRole = getEnum(Enum'ENetRole', Actor(obj).role);
		Actor(obj).role = Actor(obj).ENetRole.ROLE_Authority;
	}
	obj.setPropertyText(propName, propValue);
	if (obj.isA('Actor'))
		obj.setPropertyText("role", String(oldRole));
}

// Taken from ScrollingMessageTexture.uc
static function String replaceStr(coerce String text, coerce String match, coerce String replacement) {
	local int i;
	
	i = inStr(text, match);	

	if (i != -1)
		return left(text, i)$replacement$replaceStr(mid(Text, i + len(match)), match, replacement);
	else
		return text;
}

static function checkLog(String text, optional Pawn receiver) {
	if (receiver == None)
		log(text);
	else
		receiver.clientMessage(text);
}

static function bool isAprilFoolsDay(LevelInfo level) {
	return (level.month == 4) && (level.day == 1);
}
